#include "xpulse.h"
#include "extern.h"

void drawInfoText(char *s, int len, Window win, GC gc,
  XFontStruct *fontinfo)
{
  int y;
  int i;
  int line;
  char buffer[MAXLINELEN];
  char *ss;
  char *t;

  ss = s + len;
  y = 3 + fontinfo->ascent;
  t = buffer;
  line = 0;
  while(*s && (s < ss)) {
    if((*s == '\n') || (line >= (MAXLINELEN - 1))) {
      XDrawString(display, win, gc, 4, y, buffer, line);
      y += (fontinfo->ascent + fontinfo->descent);
      t = buffer;
      line = 0;
      s++;
    }
    else if(*s == '\t') {
      if(i = TABSIZE - (line % TABSIZE))
        for(1; i > 0; i--) {
          *t = ' ';
          t++;
	  line++;
	}
      s++;
    }
    else {
      *t++ = *s++;
      line++;
    }
  }
}

void openInfoWindow(x, y)
{
  int w;
  int h;
  int i;
  int loop;
  int bw;
  char *str;
  Window cpuwin;
  GC cpugc;
  XFontStruct *fontinfo;
  XSetWindowAttributes winattr;
  XEvent event;
  
  str = cpuinfo;
  w = 0;
  h = 0;
  i = 0;
  while(*str) {
    if(*str == '\t')
      i += (TABSIZE - (i % TABSIZE));
    else if(*str == '\n') {
      if(i > w)
        w = i;
      i = 0;
      h++;
    }
    else
      i++;
    str++;
  }
  if(w > MAXLINELEN)
    w = MAXLINELEN;
  if((fontinfo = XLoadQueryFont(display, fontname)) == NULL) {
    if((fontinfo = XLoadQueryFont(display, FONTNAME)) == NULL) {
      fontError(fontname);
      cleanUp();
      exit(EXIT_FAILURE);
    }
  }
  w = w * fontinfo->max_bounds.width + 8;
  h = h * (fontinfo->ascent + fontinfo->descent) + 8;
  if(colormode == COLOR_MONO)
    bw = 1;
  else
    bw = 0;
  x = x - (w / 2);
  y = y - (h / 2);
  if(x < 0)
    x = 0;
  if(y < 0)
    y = 0;
  if((x + w) >= screenwidth)
    x = screenheight - w;
  if((y + h) >= screenheight)
    y = screenheight - h;
  cpuwin = XCreateSimpleWindow(display, DefaultRootWindow(display),
    x, y, w, h, bw, blackpixel, whitepixel);
  cpugc = XCreateGC(display, cpuwin, 0, 0);
  winattr.override_redirect = True;
  XChangeWindowAttributes(display, cpuwin,
    CWOverrideRedirect, &winattr);
  XSelectInput(display, cpuwin,
    ExposureMask | ButtonReleaseMask | LeaveWindowMask);
  XMapRaised(display, cpuwin);
  XSetFont(display, cpugc, fontinfo->fid);
  XSetBackground(display, cpugc, whitepixel);
  if(colormode != COLOR_MONO) {
    XSetForeground(display, cpugc, greypixel);
    XFillRectangle(display, cpuwin, cpugc, 0, 0, (w + 1), (h + 1));
    XSetForeground(display, cpugc, whitepixel);
    XFillRectangle(display, cpuwin, cpugc, 0, 0, (w - 1), 2);
    XFillRectangle(display, cpuwin, cpugc, 0, 2, 2, (h - 1));
    XSetForeground(display, cpugc, greyshade[2]);
    XFillRectangle(display, cpuwin, cpugc, 1,
      (h - 2), (w - 1), (h - 1));
    XFillRectangle(display, cpuwin, cpugc,
      (w - 2), 1, (w - 1), (h - 1));
  }
  XSetForeground(display, cpugc, blackpixel);
  loop = 1;

  while(loop) {
    XNextEvent(display, &event);
    switch(event.type) {
    case Expose:
      if(event.xexpose.count == 0)
	drawInfoText(cpuinfo, strlen(cpuinfo),
	  cpuwin, cpugc, fontinfo);
      break;
    case ButtonRelease:
      if(event.xbutton.button == Button1)
        loop = 0;
      break;
    case LeaveNotify:
      loop = 0;
      break;
    }
  }
  XFreeGC(display, cpugc);
  XDestroyWindow(display, cpuwin);
  XFreeFont(display, fontinfo);
}

void readCPUinfo()
{
  int i;
  int fd;
  char *str;

  if((fd = open("/proc/cpuinfo", O_RDONLY)) < 0) {
    (void)fprintf(stderr, "%s: unable to open /proc/cpuinfo\n",
      progname);
    exit(EXIT_FAILURE);
  }
  str = cpuinfo;
  for(i = 0; i < CPUSIZE; i++)
    *str++ = '\0';
  if(read(fd, cpuinfo, (CPUSIZE - 1)) <= 0) {
    perror("reading /proc/cpuinfo");
    exit(EXIT_FAILURE);
  }
  str = cpuinfo;
  while(*str != '\0') {
    if((*str == 'c') && (*(str + 1) == 'p') && (*(str + 2) == 'u')) {
      str += 3;
      while((*str == ' ') || (*str == '\t') || (*str == ':'))
        str++;
      i = 0;
      while((*str != '\n') && (i < 8))
        cpustring[i++] = *str++;
      cpustring[i] = '\0';
      return;
    }
    else
      str++;
  }
  close(fd);
}
