#include "xpulse.h"
#include "extern.h"

/*
 *  Garbage collection.
 */

void cleanUp()
{
  modeswitch |= MODE_NOP;
  if(done & SET_BACK)
    XFreeColors(display, colormap, &bgpixel, 1, 0);
  if(done & SET_PIXFONT)
    XFreeFont(display, cpufontinfo);
  if(done & SET_GREY)
    XFreeColors(display, colormap, greyshade, 5, 0);
  if((done & SET_HIGAUGE) && (modeswitch & MODE_SHADE))
    XFreeColors(display, colormap, hicolor, MAXLOAD, 0);
  else if((done & SET_HIGAUGE) && (!(modeswitch & MODE_SHADE)))
    XFreeColors(display, colormap, hicolor, 1, 0);
  if((done & SET_LOGAUGE) && (modeswitch & MODE_SHADE))
    XFreeColors(display, colormap, locolor, MAXLOAD, 0);
  else if((done & SET_LOGAUGE) && (!(modeswitch & MODE_SHADE)))
    XFreeColors(display, colormap, locolor, 1, 0);
  if(done & SET_PIXMAP)
    XFreePixmap(display, pixmap);
  if(done & SET_PIXGC)
    XFreeGC(display, pixgc);
  if(done & SET_GAUGEGC)
    XFreeGC(display, gaugegc);
  if(done & SET_MAINGC)
    XFreeGC(display, maingc);
  if(done & SET_WINDOW)
    XDestroyWindow(display, mainwin);
  if(done & SET_DISPLAY)
    XCloseDisplay(display);
  if(done & SET_AVG);
    close(avgfd);
  if(done & SET_LOAD);
    close(loadfd);
}

/*
 *  Write warning to stderr
 */

void warn(char *msg, char *opt)
{
  if(modeswitch & MODE_WARN) {
    if(opt == NULL)
      (void)fprintf(stderr, "%s: %s\n", progname, msg);
    else
      (void)fprintf(stderr, "%s: %s: %s\n", progname, msg, opt);
  }
}

/*
 *  Unable to open font
 */

void fontError(char *name)
{
  warn("unable to open font", name);
}

/*
 *  Check if selected font exists.
 */

int checkFont(char *name)
{
  char **fontlist;
  int count;

  if((fontlist = XListFonts(display, name, 1, &count)) == NULL)
    return(False);
  XFreeFontNames(fontlist);
  if(count == 0)
    return(False);
  else
    return(True);
}

/*
 *  Color could not be allocated.
 */

void colorCellError(XColor *colorspec)
{
  (void)fprintf(stderr,
    "%s: unable to allocate color (%x %x %x)\n",
      progname, colorspec->red, colorspec->green, colorspec->blue);
  cleanUp();
  exit(EXIT_FAILURE);
}

/*
 *  Selected color mode is not supported.
 */

void colorModeError(char *reason)
{
  (void)fprintf(stderr, "%s: selected color-mode not supported",
    progname);
  if(reason != NULL)
    (void)fprintf(stderr, " (%s)\n", reason);
  else
    (void)fprintf(stderr, "\n");
  cleanUp();
  exit(EXIT_FAILURE);
}

/*
 *  Install color table for gauge meter
 */

int buildColorTable(char *colorname, unsigned long table[])
{
  int i;
  unsigned short r;
  unsigned short g;
  unsigned short b;
  unsigned short decr;
  unsigned short decg;
  unsigned short decb;
  XColor color;

  if(modeswitch & MODE_SHADE) {
    if(!XParseColor(display, colormap, colorname, &color)) {
      warn("invalid color name", colorname);
      return(0);
    }
    decr = color.red / MAXLOAD;
    decg = color.green / MAXLOAD;
    decb = color.blue / MAXLOAD;
    r = decr;
    g = decg;
    b = decb;
    for(i = 0; i < MAXLOAD; i++) {
      color.red = r;
      color.green = g;
      color.blue = b;
      if(XAllocColor(display, colormap, &color))
        table[i] = color.pixel;
      else {
	if(i > 0)
          XFreeColors(display, colormap, table, i, 0);
        colorCellError(&color);
      }
      r += decr;
      g += decg;
      b += decb;
    }
  }
  else {
    if(!XParseColor(display, colormap, colorname, &color)) {
      warn("invalid color name", colorname);
      return(0);
    }
    if(!(XAllocColor(display, colormap, &color))) {
      colorCellError(&color);
    }
    for(i = 0; i < MAXLOAD; i++)
      table[i] = color.pixel;
  }
  return(True);
}

/*
 *  Decide which color mode to use for color allocation
 *  and build color table for one meter.
 */

void buildGaugeColor(char *colorname, unsigned long table[])
{
  int i;

  while(colormode > 0) {
    switch(colormode) {
    case COLOR_MONO:
      if(modeswitch & MODE_CLASSIC) {
        modeswitch &= ~MODE_CLASSIC;
	winwidth -= 10;
      }
      for(i = 0; i < MAXLOAD; i++)
        table[i] = whitepixel;
      return;
      break;
    case COLOR_GREY:
      if(buildColorTable("white", table))
        return;
      break;
    case COLOR_MULTI:
      if(colorname != NULL)
        if(buildColorTable(colorname, table))
	  return;
      else
        if(buildColorTable("green", table))
	  return;
      break;
    default:
      colorModeError(NULL);
      break;
    }
    colormode--;
  }
}

/*
 *  Determine which color table to build
 *  depending on selected meter mode.
 */

void buildGauge()
{
  if((colormode < COLOR_MULTI) && (done & SET_HIGAUGE))
    return;
  if((done & SET_HIGAUGE) && (modeswitch & MODE_SHADE)) {
    XFreeColors(display, colormap, hicolor, MAXLOAD, 0);
    done &= ~SET_HIGAUGE;
  }
  else if((done & SET_HIGAUGE) && (!(modeswitch & MODE_SHADE))) {
    XFreeColors(display, colormap, hicolor, 1, 0);
    done &= ~SET_HIGAUGE;
  }
  if((done & SET_LOGAUGE) && (modeswitch & MODE_SHADE)) {
    XFreeColors(display, colormap, locolor, MAXLOAD, 0);
    done &= ~SET_LOGAUGE;
  }
  else if((done & SET_LOGAUGE) && (!(modeswitch & MODE_SHADE))) {
    XFreeColors(display, colormap, locolor, 1, 0);
    done &= ~SET_LOGAUGE;
  }
  /*
      Old color table is destroyed (if it already existed)
      and is ready to be rebuild.
  */
  if((modeswitch & MODE_TWIN) && (twinmode == TWIN_USERSYS)) {
    buildGaugeColor(usercolor, hicolor);
    done |= SET_HIGAUGE;
    buildGaugeColor(systemcolor, locolor);
    done |= SET_LOGAUGE;
  }
  else if((modeswitch & MODE_TWIN) && (twinmode == TWIN_AVERAGE)) {
    buildGaugeColor(totalcolor, hicolor);
    done |= SET_HIGAUGE;
    buildGaugeColor(avgcolor, locolor);
    done |= SET_LOGAUGE;
  }
  else {
    buildGaugeColor(gaugecolor, hicolor);
    done |= SET_HIGAUGE;
  }
}

/*
 *  Allocate colors for the pixmap.
 */

void initPixmapColor()
{
  int i;
  XColor grey;

  if(colormode == COLOR_MONO)
    return;
  for(i = 0; i < 5; i++) {
    grey.red = greydata[i];
    grey.green = greydata[i];
    grey.blue = greydata[i];
    if(XAllocColor(display, colormap, &grey))
      greyshade[i] = grey.pixel;
    else {
      if(i > 0)
        XFreeColors(display, colormap, greyshade, i, 0);
      colorCellError(&grey);
    }
  }
  done |= SET_GREY;
  greypixel = greyshade[4];
}

/*
 *  Draw pixmap.
 */

void initPixmap()
{
  int i;
  int x;
  int y;
  char *name;

  if(modeswitch & MODE_CLASSIC) {
    XSetBackground(display, pixgc, greypixel);
    XSetForeground(display, pixgc, greypixel);
  }
  else {
    XSetBackground(display, pixgc, bgpixel);
    XSetForeground(display, pixgc, bgpixel);
  }
  XFillRectangle(display, pixmap, pixgc, 0, 0,
    (IMAGESIZE + 1), (IMAGESIZE + 1));
  if(colormode == COLOR_MONO) {
    XSetForeground(display, pixgc, whitepixel);
    XDrawLine(display, pixmap, pixgc, 4, 3, 43, 3);
    XDrawLine(display, pixmap, pixgc, 4, 44, 43, 44);
    XDrawLine(display, pixmap, pixgc, 3, 4, 3, 43);
    XDrawLine(display, pixmap, pixgc, 44, 4, 44, 43);
    for(x = 5; x < 44; x += 2)
      XDrawPoint(display, pixmap, pixgc, x, 4);
    for(y = 5; y < 44; y += 2)
      XDrawPoint(display, pixmap, pixgc, 4, y);
    for(x = 5; x < 44; x += 2)
      XDrawPoint(display, pixmap, pixgc, x, 43);
    for(y = 5; y < 44; y += 2)
      XDrawPoint(display, pixmap, pixgc, 43, y);
    i = 1;
    for(x = 6; x < 42; x+= 2) {
      XDrawLine(display, pixmap, pixgc, x, 0, x, 2);
      if(i % 2)
        XDrawPoint(display, pixmap, pixgc, (x + 1), 1);
      else
        XDrawPoint(display, pixmap, pixgc, (x + 1), 2);
      XDrawLine(display, pixmap, pixgc, x, 45, x, 47);
      if(i % 2)
        XDrawPoint(display, pixmap, pixgc, (x + 1), 46);
      else
        XDrawPoint(display, pixmap, pixgc, (x + 1), 45);
      i++;
    }
    i = 1;
    for(y = 6; y < 42; y+= 2) {
      XDrawLine(display, pixmap, pixgc, 0, y, 2, y);
      if(i % 2)
        XDrawPoint(display, pixmap, pixgc, 1, (y + 1));
      else
        XDrawPoint(display, pixmap, pixgc, 2, (y + 1));
      XDrawLine(display, pixmap, pixgc, 45, y, 47, y);
      if(i % 2)
        XDrawPoint(display, pixmap, pixgc, 46, (y + 1));
      else
        XDrawPoint(display, pixmap, pixgc, 45, (y + 1));
      i++;
    }
  }
  else {
    XSetForeground(display, pixgc, whitepixel);
    x = 6;
    for(i = 0; i < 18; i++) {
      XDrawPoint(display, pixmap, pixgc, x, 2);
      XDrawPoint(display, pixmap, pixgc, x, 45);
      XDrawPoint(display, pixmap, pixgc, 2, x);
      XDrawPoint(display, pixmap, pixgc, 45, x);
      x += 2;
    }
    XSetForeground(display, pixgc, greyshade[4]);
    x = 6;
    for(i = 0; i < 18; i++) {
      XDrawPoint(display, pixmap, pixgc, x, 1);
      XDrawPoint(display, pixmap, pixgc, x, 46);
      XDrawPoint(display, pixmap, pixgc, 1, x);
      XDrawPoint(display, pixmap, pixgc, 46, x);
      x += 2;
    }
    XSetForeground(display, pixgc, greyshade[3]);
    x = 6;
    for(i = 0; i < 18; i++) {
      XDrawPoint(display, pixmap, pixgc, x, 0);
      XDrawPoint(display, pixmap, pixgc, x, 47);
      XDrawPoint(display, pixmap, pixgc, 0, x);
      XDrawPoint(display, pixmap, pixgc, 47, x);
      x += 2;
    }

    XSetForeground(display, pixgc, greyshade[2]);
    x = 7;
    for(i = 0; i < 18; i++) {
      XDrawPoint(display, pixmap, pixgc, x, 2);
      XDrawPoint(display, pixmap, pixgc, x, 45);
      XDrawPoint(display, pixmap, pixgc, 2, x);
      XDrawPoint(display, pixmap, pixgc, 45, x);
      x += 2;
    }
    XSetForeground(display, pixgc, greyshade[1]);
    x = 7;
    for(i = 0; i < 18; i++) {
      XDrawPoint(display, pixmap, pixgc, x, 1);
      XDrawPoint(display, pixmap, pixgc, x, 46);
      XDrawPoint(display, pixmap, pixgc, 1, x);
      XDrawPoint(display, pixmap, pixgc, 46, x);
      x += 2;
    }
    XSetForeground(display, pixgc, greyshade[0]);
    x = 7;
    for(i = 0; i < 18; i++) {
      XDrawPoint(display, pixmap, pixgc, x, 0);
      XDrawPoint(display, pixmap, pixgc, x, 47);
      XDrawPoint(display, pixmap, pixgc, 0, x);
      XDrawPoint(display, pixmap, pixgc, 47, x);
      x += 2;
    }

    XSetForeground(display, pixgc, greyshade[0]);
    XFillRectangle(display, pixmap, pixgc, 3, 3, 42, 42);
    XSetForeground(display, pixgc, greyshade[2]);
    XDrawRectangle(display, pixmap, pixgc, 4, 4, 39, 39);
    XSetForeground(display, pixgc, greyshade[1]);
    XDrawRectangle(display, pixmap, pixgc, 5, 5, 37, 37);

    XSetForeground(display, pixgc, blackpixel);
    XDrawPoint(display, pixmap, pixgc, 3, 3);
    XDrawPoint(display, pixmap, pixgc, 3, 44);
    XDrawPoint(display, pixmap, pixgc, 44, 3);
    XDrawPoint(display, pixmap, pixgc, 44, 44);
  }
  XSetForeground(display, pixgc, whitepixel);

  cpufontinfo = NULL;
  for(i = 0; (propfontlist[i] != NULL) && (cpufontinfo == NULL); i++) {
    name = (char *)propfontlist[i];
    if(checkFont(name) == True) {
      if((cpufontinfo = XLoadQueryFont(display, name)) == NULL) {
        fontError(NULL);
	cleanUp();
	exit(EXIT_FAILURE);
      }
    }
  }
  if(cpufontinfo == NULL) {
    fontError(NULL);
    cleanUp();
    exit(EXIT_FAILURE);
  }
  y = (IMAGESIZE / 2) + (cpufontinfo->ascent / 2);
  i = strlen(cpustring);
  x = XTextWidth(cpufontinfo, cpustring, i);
  x = (IMAGESIZE / 2) - (x / 2);
  XSetFont(display, pixgc, cpufontinfo->fid);
  done |= SET_PIXFONT;
  XDrawString(display, pixmap, pixgc, x, y, cpustring, i);
}

/*
 *  Out of memory error.
 */

void memError()
{
  (void)fprintf(stderr, "%s: unable to allocate memory\n", progname);
  cleanUp();
  exit(EXIT_FAILURE);
}

/*
 *  Main resource installer/allocater.
 */

void initApp()
{
  Window rootwin;
  XWMHints *wmhints;
  XSizeHints *sizehints;
  XClassHint *classhint;
  XTextProperty winname;
  XTextProperty iconname;
  XColor colorspec;
  int depth;

  readCPUinfo();
  if((loadfd = open("/proc/stat", O_RDONLY)) < 0) {
    (void)fprintf(stderr, "%s: unable to open /proc/stat\n", progname);
    cleanUp();
    exit(EXIT_FAILURE);
  }
  done |= SET_LOAD;
  if((avgfd = open("/proc/loadavg", O_RDONLY)) < 0) {
    (void)fprintf(stderr, "%s: unable to open /proc/loadavg\n",
      progname);
    cleanUp();
    exit(EXIT_FAILURE);
  }
  done |= SET_AVG;
  if(modeswitch & MODE_CLASSIC)
    winwidth += 10;
  screen = DefaultScreen(display);
  screenwidth = DisplayWidth(display, screen);
  screenheight = DisplayHeight(display, screen);
  visual = DefaultVisual(display, screen);
  colormap = DefaultColormap(display, screen);
  rootwin = DefaultRootWindow(display);
  depth = DefaultDepth(display, screen);
  blackpixel = BlackPixel(display, screen);
  whitepixel = WhitePixel(display, screen);
  /*
    check visual type
  */
  if((colormode == COLOR_MULTI)
      && (modeswitch & MODE_COLORSET)
      && ((visual->class == GrayScale)
      || (visual->class == StaticGray)))
    colorModeError("gray visual");
  if((colormode >= COLOR_GREY)
      && (depth < 4)
      && (modeswitch & MODE_COLORSET))
    colorModeError("not enough color planes");
  if((colormode == COLOR_MONO) && (modeswitch & MODE_CLASSIC))
    colorModeError("classic-mode");
  if((visual->class == GrayScale) || (visual->class == StaticGray)) {
    if(depth < 4)
      colormode = COLOR_MONO;
    else
      colormode = COLOR_GREY;
  }
  pixmap = XCreatePixmap(display, rootwin,
    IMAGESIZE, IMAGESIZE, depth);
  pixgc = XCreateGC(display, pixmap, 0, 0);
  done |= (SET_PIXMAP | SET_PIXGC);

  if(!(sizehints = XAllocSizeHints()))
    memError();
  if(!(wmhints = XAllocWMHints()))
    memError();
  if(!(classhint = XAllocClassHint()))
    memError();
  sizehints->flags = PMinSize | PMaxSize;
  sizehints->min_width = winwidth;
  sizehints->min_height = winheight;
  sizehints->max_width = winwidth;
  sizehints->max_height = winheight;
  if(modeswitch & MODE_USERPOS) {
    sizehints->x = winx;
    sizehints->y = winy;
    sizehints->flags |= USPosition;
  }
  if(modeswitch & MODE_ICONIC)
    wmhints->initial_state = IconicState;
  else
    wmhints->initial_state = NormalState;
  wmhints->input = True;
  wmhints->icon_pixmap = pixmap;
  wmhints->flags = (StateHint | InputHint | IconPixmapHint);
  if(!XStringListToTextProperty(&deftitle, 1, &winname))
    memError();
  if(!XStringListToTextProperty(&deftitle, 1, &iconname))
    memError();
  classhint->res_name = progname;
  classhint->res_class = deftitle;
  mainwin = XCreateSimpleWindow(display, rootwin,
    winx, winy, sizehints->min_width, sizehints->min_height,
    0, whitepixel, blackpixel);
  done |= SET_WINDOW;
  XSetWMProperties(display, mainwin, &winname, &iconname,
    NULL, 0, sizehints, wmhints, classhint);

  maingc = XCreateGC(display, mainwin, 0, 0);
  gaugegc = XCreateGC(display, mainwin, 0, 0);
  done |= (SET_MAINGC | SET_GAUGEGC);
  wmprot = XInternAtom(display, "WM_PROTOCOLS", 0);
  wmdel = XInternAtom(display, "WM_DELETE_WINDOW", 0);
  XSetWMProtocols(display, mainwin, &wmdel, 1);
  if(strcmp(bgcolor, "black") == 0) {
    bgpixel = blackpixel;
  }
  else {
    if(XParseColor(display, colormap, bgcolor, &colorspec) == False) {
      warn("invalid color name", bgcolor);
      bgpixel = blackpixel;
    }
    else {
      if(XAllocColor(display, colormap, &colorspec) == False) {
        warn("unable to allocate background color", bgcolor);
        bgpixel = blackpixel;
      }
      else {
	bgpixel = colorspec.pixel;
        done |= SET_BACK;
      }
    }
  }
  XSetWindowBackground(display, mainwin, bgpixel);
  buildGauge();
  initPixmapColor();
  initPixmap();
  XSelectInput(display, mainwin, (ButtonPressMask
    | ButtonReleaseMask | KeyPressMask | ExposureMask));
  XMapRaised(display, mainwin);
}
