/*
    xpulse.c
    System load gauge tool
    (c)1996,97,98 P.Murmann
*/

#include "xpulse.h"

unsigned short greydata[] = { 0x2000, 0x4000, 0x6000, 0xa000, 0xc000 };
const char *propfontlist[] = {
  "-adobe-helvetica-medium-r-normal--10-100-75-75-p-56-iso8859-1",
  "-adobe-utopia-regular-r-normal--10-100-75-75-p-56-iso8859-1",
  "-b&h-lucida-medium-i-normal-sans-10-100-75-75-p-59-iso8859-1",
  "-bitstream-charter-medium-r-normal--0-0-75-75-p-0-iso8859-1",
  "fixed", "6x10", "6x13", "6x9", "7x13", "7x14", NULL};

unsigned long modeswitch;
char *progname = NULL;
int cmdc;
char **cmdv;
char filebuffer[BUFSIZ];
char cpuinfo[CPUSIZE];
char cpustring[8];
int loadfd;
int avgfd;
int winx = 0;
int winy = 0;
int colormode = COLOR_MULTI;
int twinmode;
int screenwidth;
int screenheight;
int winwidth = OFFSET + IMAGESIZE + 6 + 100 + OFFSET;
int winheight = OFFSET + IMAGESIZE + OFFSET;
unsigned long blackpixel;
unsigned long whitepixel;
unsigned long greypixel;
unsigned long bgpixel;
char dispname[256];
char fontname[256];
char geometry[20];
Display *display;
Window mainwin;
GC maingc;
GC gaugegc;
GC pixgc;
Pixmap pixmap;
int screen;
Visual *visual;
Colormap colormap;
Atom wmprot;
Atom wmdel;
char *deftitle = "xpulse";
XFontStruct *cpufontinfo;
XFontStruct *fixfontinfo;
unsigned long hicolor[MAXLOAD];
unsigned long locolor[MAXLOAD];
char bgcolor[32];
char gaugecolor[32];
char usercolor[32];
char systemcolor[32];
char totalcolor[32];
char avgcolor[32];
unsigned long greyshade[5];
unsigned long done;
int multiply;

extern void parseOpt();
extern void initApp();
extern void timerLoop(int);
extern void mainLoop();
extern void cleanUp();
extern void warn(char *, char *);

void gotSig(int sig)
{
  warn("caught SIGTERM/SIGINT", NULL);
  cleanUp();
  exit(EXIT_SUCCESS);
}

void initHandler()
{
  struct sigaction act;
  struct itimerval itv;

  act.sa_flags = 0;
  sigemptyset(&act.sa_mask);
  act.sa_handler = timerLoop;
  if(sigaction(SIGALRM, &act, NULL) != 0) {
    (void)fprintf(stderr, "%s: unable to set irq\n", progname);
    cleanUp();
    exit(EXIT_FAILURE);
  }
  itv.it_value.tv_sec = 0;
  itv.it_value.tv_usec = SLEEPTIME;
  itv.it_interval.tv_sec = 0;
  itv.it_interval.tv_usec = SLEEPTIME;
  if(setitimer(ITIMER_REAL, &itv, NULL) < 0) {
    (void)fprintf(stderr, "%s: unable to set timer\n", progname);
    cleanUp();
    exit(EXIT_FAILURE);
  }
  if((signal(SIGTERM, gotSig) == SIG_ERR)
      || (signal(SIGINT, gotSig) == SIG_ERR)) {
    (void)fprintf(stderr, "%s: unable to install signal handler\n",
      progname);
    cleanUp();
    exit(EXIT_FAILURE);
  }
}

int main(int argc, char *argv[])
{
  progname = argv[0];
  cmdv = argv;
  cmdc = argc;
  done = 0L;

  parseInput();
  initApp();
  initHandler();
  mainLoop();
  cleanUp();
  exit(EXIT_SUCCESS);
}
