#include "xpulse.h"
#include "extern.h"

extern void warn(char *, char*);

/*
 *  Display command line usage.
 */

void writeUsage()
{
  (void)fprintf(stderr, "xpulse 0.2.3\nusage: %s [options]\n"
    "-display *   set display\n"
    "-geometry *  set geometry\n"
    "-fg *        set foreground\n"
    "-bg *        set background\n"
    "-font *      set font\n"
    "-quiet       suppress warnings\n"
    "-classic     enable classic mode\n"
    "-iconic      iconic startup\n"
    "-twin        enable twin gauge\n"
    "-switch      enable twin-mode switching\n"
    "-noswitch    disable twin-mode switching\n"
    "-usersys     user/system twin mode\n"
    "-totalavg    total/average twin mode\n"
    "-color       use colored display\n"
    "-gr[ae]y     use grey scale\n"
    "-mono        use black/white display\n"
    "-noshade     prevent gauge shading\n"
    "-user *      set user color\n"
    "-system *    set system color\n"
    "-total *     set total color\n"
    "-average *   set average color\n",
    progname);
  exit(EXIT_FAILURE);
}

/*
 *  copy string
 */

int copyString(char *src, char *dest, int len)
{
  if(strlen(src) < len)
    (void)strcpy(dest, src);
  else
    (void)strncpy(dest, src, (size_t)len);
}

/*
 *  Extract resource data base
 */

XrmDatabase globdb;

void extractRDB()
{
  XrmDatabase homedb;
  XrmDatabase servdb;
  XrmDatabase appdb;
  XrmValue value;
  int i;
  int a;
  int b;
  char *p;
  char name[BUFSIZ];
  char *str_type[20];

  XrmInitialize();
  (void)strcpy(name, APPSTRING);
  appdb = XrmGetFileDatabase(name);
  (void)XrmMergeDatabases(appdb, &globdb);
  if(XResourceManagerString(display) != NULL)
    servdb = XrmGetStringDatabase(XResourceManagerString(display));
  else {
    if((p = getenv("HOME")) != NULL) {
      (void)strcpy(name, (const char *)p);
      (void)strcat(name, "/.Xdefaults");
      servdb = XrmGetFileDatabase(name);
      XrmMergeDatabases(servdb, &globdb);
    }
  }
  name[0] = '\0';
  if((p = getenv("XENVIRONMENT")) == NULL) {
    if((p = getenv("HOME")) != NULL) {
      (void)strcpy(name, (const char *)p);
      (void)strcat(name, "/.Xdefaults-");
      i = strlen(name);
      (void)gethostname(&name[i], (BUFSIZ - i));
    }
  }
  if(name[0]) {
    homedb = XrmGetFileDatabase(name);
    XrmMergeDatabases(homedb, &globdb);
  }

  /*
     read values
  */

  if((XrmGetResource(globdb, "xpulse.geometry", "Xpulse.Geometry",
      str_type, &value) == True)
      && (geometry[0] == '\0')) {
    (void)strncpy(geometry, value.addr, (int)value.size);
  }
  if(geometry[0]) {
    i = XParseGeometry(geometry, &winx, &winy, &a, &b);
    if((i & XValue) || (i & YValue))
      modeswitch |= MODE_USERPOS;
  }
  if(XrmGetResource(globdb, "xpulse.fg", "Xpulse.Fg",
      str_type, &value) == True) {
    (void)copyString(value.addr, gaugecolor, 32);
  }
  if(XrmGetResource(globdb, "xpulse.bg", "Xpulse.Bg",
      str_type, &value) == True) {
    (void)copyString(value.addr, bgcolor, 32);
  }
  if((XrmGetResource(globdb, "xpulse.font", "Xpulse.Font",
      str_type, &value) == True)
      && (fontname[0] == '\0')) {
    (void)strncpy(fontname, value.addr, (int)value.size);
  }
  if(XrmGetResource(globdb, "xpulse.warnState", "Xpulse.WarnState",
      str_type, &value) == True) {
    (void)strncpy(name, value.addr, (int)value.size);
    if(strcasecmp(name, "True") == 0)
      modeswitch |= MODE_WARN;
    else if(strcasecmp(name, "False") == 0)
      modeswitch &= ~MODE_WARN;
  }
  if(XrmGetResource(globdb, "xpulse.debugState", "Xpulse.DebugState",
      str_type, &value) == True) {
    (void)strncpy(name, value.addr, (int)value.size);
    if(strcasecmp(name, "True") == 0)
      modeswitch |= MODE_DEBUG;
    else if(strcasecmp(name, "False") == 0)
      modeswitch &= ~MODE_DEBUG;
  }
  if(XrmGetResource(globdb, "xpulse.switchState", "Xpulse.SwitchState",
      str_type, &value) == True) {
    (void)strncpy(name, value.addr, (int)value.size);
    if(strcasecmp(name, "True") == 0)
      modeswitch |= MODE_SWITCH;
    else if(strcasecmp(name, "False") == 0)
      modeswitch &= ~MODE_SWITCH;
  }
  if(XrmGetResource(globdb, "xpulse.classicState",
      "Xpulse.ClassicState", str_type, &value) == True) {
    (void)strncpy(name, value.addr, (int)value.size);
    if(strcasecmp(name, "True") == 0)
      modeswitch |= MODE_CLASSIC;
    else if(strcasecmp(name, "False") == 0)
      modeswitch &= ~MODE_CLASSIC;
  }
  if(XrmGetResource(globdb, "xpulse.iconicState", "Xpulse.IconicState",
      str_type, &value) == True) {
    (void)strncpy(name, value.addr, (int)value.size);
    if(strcasecmp(name, "True") == 0)
      modeswitch |= MODE_ICONIC;
    else if(strcasecmp(name, "False") == 0)
      modeswitch &= ~MODE_ICONIC;
  }
  if(XrmGetResource(globdb, "xpulse.shadeState", "Xpulse.ShadeState",
      str_type, &value) == True) {
    (void)strncpy(name, value.addr, (int)value.size);
    if(strcasecmp(name, "True") == 0)
      modeswitch |= MODE_SHADE;
    else if(strcasecmp(name, "False") == 0)
      modeswitch &= ~MODE_SHADE;
  }
  if(XrmGetResource(globdb, "xpulse.twinState", "Xpulse.TwinState",
      str_type, &value) == True) {
    (void)strncpy(name, value.addr, (int)value.size);
    if(strcasecmp(name, "True") == 0)
      modeswitch |= MODE_TWIN;
    else if(strcasecmp(name, "False") == 0)
      modeswitch &= ~MODE_TWIN;
  }
  if(XrmGetResource(globdb, "xpulse.twinMode", "Xpulse.TwinMode",
      str_type, &value) == True) {
    (void)strncpy(name, value.addr, (int)value.size);
    if(strcasecmp(name, "usersys") == 0)
      twinmode = TWIN_USERSYS;
    else if(strcasecmp(name, "totalavg") == 0)
      twinmode = TWIN_AVERAGE;
  }
  if(XrmGetResource(globdb, "xpulse.meterState", "Xpulse.MeterState",
      str_type, &value) == True) {
    (void)strncpy(name, value.addr, (int)value.size);
    if(strcasecmp(name, "True") == 0)
      modeswitch |= MODE_METER;
    else if(strcasecmp(name, "False") == 0)
      modeswitch &= ~MODE_METER;
  }
  if(XrmGetResource(globdb, "xpulse.colorMode", "Xpulse.ColorMode",
      str_type, &value) == True) {
    (void)strncpy(name, value.addr, (int)value.size);
    if(strcasecmp(name, "color") == 0)
      colormode = COLOR_MULTI;
    else if(strcasecmp(name, "grey") == 0)
      colormode = COLOR_GREY;
    else if(strcasecmp(name, "gray") == 0)
      colormode = COLOR_GREY;
    else if(strcasecmp(name, "mono") == 0)
      colormode = COLOR_MONO;
  }
  if(XrmGetResource(globdb, "xpulse.gaugeColor", "Xpulse.GaugeColor",
      str_type, &value) == True) {
    (void)copyString(value.addr, gaugecolor, 32);
  }
  if(XrmGetResource(globdb, "xpulse.userColor", "Xpulse.UserColor",
      str_type, &value) == True) {
    (void)copyString(value.addr, usercolor, 32);
  }
  if(XrmGetResource(globdb, "xpulse.systemColor", "Xpulse.SystemColor",
      str_type, &value) == True) {
    (void)copyString(value.addr, systemcolor, 32);
  }
  if(XrmGetResource(globdb, "xpulse.totalColor", "Xpulse.TotalColor",
      str_type, &value) == True) {
    (void)copyString(value.addr, totalcolor, 32);
  }
  if(XrmGetResource(globdb, "xpulse.avgColor", "Xpulse.AvgColor",
      str_type, &value) == True) {
    (void)copyString(value.addr, avgcolor, 32);
  }
}

/*
 * Parse command line options.
 */

void extractOpt()
{
  int i;
  char *arg;

  for(i = 1; i < cmdc; i++) {
    arg = cmdv[i];
    if(*arg != '-')
      writeUsage();
    arg++;
    if((strcmp(arg, "display") == 0) && ((i + 1) < cmdc)) {
      i++;
      continue;
    }
    else if((strcmp(arg, "geometry") == 0) && ((i + 1) < cmdc)) {
      i++;
      copyString(cmdv[i], geometry, 20);
      continue;
    }
    else if((strcmp(arg, "fg") == 0) && ((i + 1) < cmdc)) {
      i++;
      copyString(cmdv[i], gaugecolor, 32);
      continue;
    }
    else if((strcmp(arg, "bg") == 0) && ((i + 1) < cmdc)) {
      i++;
      copyString(cmdv[i], bgcolor, 32);
      continue;
    }
    else if((strcmp(arg, "font") == 0) && ((i + 1) < cmdc)) {
      i++;
      copyString(cmdv[i], fontname, 256);
      continue;
    }
    else if(strcmp(arg, "help") == 0) {
      writeUsage();
    }
    else if(strcmp(arg, "quiet") == 0) {
      modeswitch &= ~MODE_WARN;
      continue;
    }
    else if(strcmp(arg, "debug") == 0) {
      modeswitch |= MODE_DEBUG;
      continue;
    }
    else if(strcmp(arg, "switch") == 0) {
      modeswitch |= MODE_SWITCH;
      continue;
    }
    else if(strcmp(arg, "noswitch") == 0) {
      modeswitch &= ~MODE_SWITCH;
      continue;
    }
    else if(strcmp(arg, "classic") == 0) {
      modeswitch |= MODE_CLASSIC;
      continue;
    }
    else if(strcmp(arg, "iconic") == 0) {
      modeswitch |= MODE_ICONIC;
      continue;
    }
    else if(strcmp(arg, "shade") == 0) {
      modeswitch |= MODE_SHADE;
      continue;
    }
    else if(strcmp(arg, "noshade") == 0) {
      modeswitch &= ~MODE_SHADE;
      continue;
    }
    else if(strcmp(arg, "meter") == 0) {
      modeswitch |= MODE_METER;
      continue;
    }
    else if(strcmp(arg, "nometer") == 0) {
      modeswitch &= ~MODE_METER;
      continue;
    }
    else if(strcmp(arg, "twin") == 0) {
      modeswitch |= MODE_TWIN;
      continue;
    }
    else if(strcmp(arg, "single") == 0) {
      modeswitch &= ~MODE_TWIN;
      continue;
    }
    else if(strcmp(arg, "totalavg") == 0) {
      twinmode = TWIN_AVERAGE;
      continue;
    }
    else if(strcmp(arg, "usersys") == 0) {
      twinmode = TWIN_USERSYS;
      continue;
    }
    else if(strcmp(arg, "color") == 0) {
      colormode = COLOR_MULTI;
      modeswitch |= MODE_COLORSET;
      continue;
    }
    else if(strcmp(arg, "grey") == 0) {
      colormode = COLOR_GREY;
      modeswitch |= MODE_COLORSET;
      continue;
    }
    else if(strcmp(arg, "gray") == 0) {
      colormode = COLOR_GREY;
      modeswitch |= MODE_COLORSET;
      continue;
    }
    else if(strcmp(arg, "mono") == 0) {
      colormode = COLOR_MONO;
      modeswitch |= MODE_COLORSET;
      continue;
    }
    else if((strcmp(arg, "user") == 0) && ((i + 1) < cmdc)) {
      i++;
      copyString(cmdv[i], usercolor, 32);
      continue;
    }
    else if((strcmp(arg, "system") == 0) && ((i + 1) < cmdc)) {
      i++;
      copyString(cmdv[i], systemcolor, 32);
      continue;
    }
    else if((strcmp(arg, "total") == 0) && ((i + 1) < cmdc)) {
      i++;
      copyString(cmdv[i], totalcolor, 32);
      continue;
    }
    else if((strcmp(arg, "average") == 0) && ((i + 1) < cmdc)) {
      i++;
      copyString(cmdv[i], avgcolor, 32);
      continue;
    }
    else {
      /*
      arg--;
      (void)fprintf(stderr, "%s: unknown option: %s\n", progname, arg);
      */
      writeUsage();
    }
  }
}

extern int checkFont(char *);
extern void fontError(char *);

/*
 *  Collect user options (Xrdb and command line)
 */

void parseInput()
{
  int i;
  char *arg;
  char *dispname;

  dispname = NULL;
  modeswitch = MODE_DEFAULT;
  twinmode = TWIN_DEFAULT;
  colormode = COLOR_MULTI;
  geometry[0] = '\0';
  multiply = MAXLOAD + 1;

  (void)strcpy(fontname, FONTNAME);
  (void)strcpy(bgcolor, "black");
  (void)strcpy(gaugecolor, "green");
  (void)strcpy(totalcolor, "cyan");
  (void)strcpy(usercolor, "blue");
  (void)strcpy(systemcolor, "red");
  (void)strcpy(avgcolor, "yellow");

  for(i = 1; i < cmdc; i++) {
    arg = cmdv[i];
    if(*arg == '-') {
      arg++;
      if((strcmp(arg, "display") == 0) && ((i + 1) < cmdc))
        dispname = cmdv[i + 1];
    }
  }
  if(!(display = XOpenDisplay(dispname))) {
    (void)fprintf(stderr, "%s: unable to open display '%s'\n",
      progname, dispname);
    exit(EXIT_FAILURE);
  }
  extractRDB();
  extractOpt();

  if((modeswitch & MODE_CLASSIC) && (modeswitch & MODE_TWIN)) {
    warn("classic- and twin-mode exclude each other", NULL);
    cleanUp();
    exit(EXIT_FAILURE);
  }
  if((modeswitch & MODE_SWITCH) && (!(modeswitch & MODE_TWIN))) {
    warn("switch-mode depends on twin-mode", NULL);
    cleanUp();
    exit(EXIT_FAILURE);
  }
  if(checkFont(fontname) == False) {
    fontError(fontname);
    cleanUp();
    exit(EXIT_FAILURE);
  }
}
