/*
    xpulse.h
*/

#ifndef _XPULSE_H
#define _XPULSE_H

#include <errno.h>
#include <fcntl.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#include <X11/X.h>
#include <X11/Xlib.h>
#include <X11/Xresource.h>
#include <X11/Xutil.h>

#define IMAGESIZE 48
#define OFFSET 2
#define GAUGEHEIGHT 16
#define MAXLOAD 10
#define CPUSIZE 512
#define FONTNAME "6x10"
#define MAXLINELEN 256
#define TABSIZE 8
#define SLEEPTIME 250000
#define APPSTRING "/usr/lib/X11/app-defaults/Xpulse"

#define MODE_DEBUG	(1L << 0)
#define MODE_CLASSIC	(1L << 1)
#define MODE_ICONIC	(1L << 2)
#define MODE_TWIN	(1L << 3)
#define MODE_USERPOS	(1L << 4)
#define MODE_SHADE	(1L << 5)
#define MODE_METER	(1L << 6)
#define MODE_WARN	(1L << 7)
#define MODE_SWITCH	(1L << 8)
#define MODE_COLORSET	(1L << 9)
#define MODE_NOP	(1L << 31)
#define MODE_DEFAULT	(MODE_SHADE | MODE_WARN)

#define COLOR_MONO	1
#define COLOR_GREY	2
#define COLOR_MULTI	3
#define COLOR_DEFAULT	COLOR_MULTI

#define SET_LOAD	(1L << 0)
#define SET_AVG		(1L << 1)
#define SET_DISPLAY	(1L << 2)
#define SET_WINDOW	(1L << 3)
#define SET_MAINGC	(1L << 4)
#define SET_GAUGEGC	(1L << 5)
#define SET_PIXGC	(1L << 6)
#define SET_PIXMAP	(1L << 7)
#define SET_HIGAUGE	(1L << 8)
#define SET_LOGAUGE	(1L << 9)
#define SET_GREY	(1L << 10)
#define SET_PIXFONT	(1L << 11)
#define SET_CHILD	(1L << 12)
#define SET_BACK	(1L << 13)

#define TWIN_USERSYS	1
#define TWIN_AVERAGE	2
#define TWIN_DEFAULT	TWIN_USERSYS

typedef struct {
  unsigned long user;
  unsigned long system;
  unsigned long nice;
  unsigned long idle;
  unsigned long total;
  unsigned long average;
} Load;

extern void cleanUp();
extern void initColors();
extern void initPixmap();
extern void memError();
extern void initApp();
extern void writeUsage();
extern int copyString();
extern int checkFont();
extern void extractOpt();
extern void parseOpt();
extern void fontError();
extern void readCPUinfo();

#endif
